import configparser # for .ini files
import numpy as np
import re
import ast


class ConfigObject_PhRetrieval():

    def __init__(self, configfile='./AnalysisConfig_default.ini'):
        self.config = configparser.ConfigParser()
        self.config.optionxform = lambda option: option  # preserves case in key names
        self.config.read(configfile, encoding='utf-8-sig')

        self.get_section_Ph_Retrieval()
        

    def get_section_Ph_Retrieval(self):
        section_title= 'Ph_Retrieval'
        self.Ph_Retrieval={}
        self.Ph_Retrieval['Lambda']=self.str2array(self.config[section_title]['Lambda'])
        self.Ph_Retrieval['Px']    =self.str2array(self.config[section_title]['Px'])
        self.Ph_Retrieval['TLens'] =self.str2array(self.config[section_title]['TLens'])
        self.Ph_Retrieval['Na']    =self.str2array(self.config[section_title]['Na'])
        self.Ph_Retrieval['n']     =self.str2array(self.config[section_title]['n'])
        self.Ph_Retrieval['Mag']   =self.str2array(self.config[section_title]['Mag'])
        self.Ph_Retrieval['Mag2']  =self.str2array(self.config[section_title]['Mag2'])
        self.Ph_Retrieval['NM']    =self.str2array(self.config[section_title]['NM']).astype('int')
        self.Ph_Retrieval['zssize']=self.str2array(self.config[section_title]['zssize'])
        self.Ph_Retrieval['flens'] =self.Ph_Retrieval['TLens']/self.Ph_Retrieval['Mag']

    def PeriodToSpeed(self,s):
        r=1/s
        return r
    
    def topower(self,s):
        r=s[0]*2**s[1]
        return r
    
    def str2array(self, s):
        # Remove space after [
        s=re.sub('\[ +', '[', s.strip())
        # Replace commas and spaces
        s=re.sub('[,\s]+', ', ', s)
        # Handle inf by using a place holder that is a big number 2**32=4294967296
        s=re.sub('inf', '4294967296', s)

        r=np.array(ast.literal_eval(s))  # literal_eval is a safe way to convert a string to a python data type, it will not evaluate commands.
        
        # Extchange the large place holder with propper inf
        r=np.where(r==2**32, np.inf, r)
        r=np.where(r==-2**32, -np.inf, r)
        
        #
        if r.size==1:
            r=np.double(r)

        return r


    def GetSection(self, section_title):
        Section = {}
        for key in self.config[section_title]:
            Section[key] = eval(self.config[section_title][key])
        
        return Section



##### Class test code ##################
def main():
    # Path prefix for local data storage
    configfile = "./AnalysisConfig_default.ini"
    config=ConfigObject(configfile)
    config.get_section_PosCalibration()    
    config.get_section_DataSelectionRules()   
    config.get_section_TrajectorySelectionRules()

if __name__ == '__main__':
    main()
