import numpy as np
from datetime import datetime
from os import listdir
import h5py
import json
import pickle 
import scipy.io as io
from PIL import Image

class DataFileIO_Lib():
    
    def GetScanData(self, ScanProject):
        """
            Opens files and returns the content in the right format
        """
        PathToFile = ScanProject.Files['ScanData']
        fid = open(PathToFile,'r')
        data_type = np.dtype ('uint64').newbyteorder ('>')
        Data = np.fromfile (fid, dtype=data_type)
        fid.close()
        return Data

    def SheckFileExist(self,folder,searchstring, startswith='', endswith=''):
        """
        Looks throuhg the folder for the files with the name content "searchstring" and ends with .pkl
        output is how many of the .pkl files contained the string "searchstring" in its name 

        """
        dirContent=listdir(folder)
        res=0
        for C in dirContent:
            if C.endswith(endswith) and C.startswith(startswith):
                r=str.find(C,searchstring)
                if r!=-1:
                    res+=1
        return res

    def PickleData(self, ScanProject, override=1,folder='',FileEnd='_ScanProject.pkl'):
        """
        Saves the workspace to a file in the project folder with the name date_num_ScanProject.pkl'
        override:  if 0 makes a new file for each save (changing the num part of the file name) 
                   if 1 overides the last file, (finds the last num and overwrite)   
        """
        if folder=='':
            folder=ScanProject.Files['AnalysisConfig'][:-18]
        
        today=datetime.today().strftime('%Y-%m-%d')
        res=self.SheckFileExist(folder,'', startswith=today, endswith=FileEnd)    
        
        file= today + '_' + str(res) + FileEnd
        if override:
            if res!=0:
                file= today + '_' + str(res-1) + FileEnd
                print("OBS! Data found, will be overwitten")  

        with open(folder + file,'wb') as pickle_file:
            """
            There are 5 pickle protocols in current version when writing (look with pickle.HIGHEST_PROTOCOL)
            protocol = 
            0:  Human readable 
            1,2:  Old binary format 
            3: Added for Python 3.0, is default  
            4: Added for Python 3.4, adds support for very large objects (is defaut in python 3.8)
            (5): is avalable in python 3.8 not avalable here
            """
            pickle.dump(ScanProject, pickle_file, protocol=4)            

        print("Data is pickled")

    def UnPickleData(self, ScanProject=None, folder='', file=''):
        """
        if file is not included then it will find the lates data to open in the project folder
        OBS! this will override the current ScanProject 
             Run: ScanProject=DataObject(data_path_handler, FILE_TO_OPEN) to get minimal ScanProject 
                            
        """
        if folder=='':
            folder=ScanProject.Files['AnalysisConfig'][:-18]

        if file=='':
            LatestDate=datetime(2000,1,1,0,0,0,0) #microseconds are used to label different runs over the same day, thus max 1000 per day
            dirContent=listdir(folder)
            for C in dirContent:
                if C.endswith('_ScanProject.pkl'):
                    Cindex1=str.find(C,'_')
                    Cindex2=str.find(C,'_ScanProject.pkl')
                    n=np.array([C[Cindex1+1:Cindex2]]).astype(np.int32)[0]
                    date=C[:Cindex1]
                    dash=str.find(date,'-')
                    year=int(date[:dash])
                    date=date[dash+1:]
                    dash=str.find(date,'-')
                    month=int(date[:dash])
                    day=int(date[dash+1:])
                    date=datetime(year,month,day,0,0,0,n)
                    if LatestDate<=date:
                        LatestDate=date
                        file=C
            if file=='':
                print('No file to open in folder')
        else:
            res=self.SheckFileExist(folder,file)
            if res==0:
                print('No file with name:' + file + ' in folder')
            
        if file!='':
            print('Loading file: ' + file)
            with open(folder + file,'rb') as pickle_file:
                ScanProject_res = pickle.load(pickle_file)
        
        print('Load successful')
        return ScanProject_res
        
    def readHDF5(self, ScanProject, folder='', file=''):
        """
        if file is not included then it will find the lates data to open in the project folder
        OBS! this will override the current ScanProject 
             Run: ScanProject=DataObject(data_path_handler, FILE_TO_OPEN) to get minimal ScanProject 
                            
        """
        if folder=='':
            folder=ScanProject.Files['AnalysisConfig'][:-18]

        if file=='':
            num=0
            dirContent=listdir(folder)
            for C in dirContent:
                if C.endswith('_ScanProject.hdf5'):
                    Cindex1=str.find(C,'_')
                    Cindex2=str.find(C,'_ScanProject.hdf5')
                    n=np.array([C[Cindex1+1:Cindex2]]).astype(np.int32)[0]
                    if n>=num:
                        num=n
                        file=C
            if file=='':
                print('No file to open in folder')
        else:
            res=SheckFileExist(folder,file)
            if res==0:
                print('No file with name:' + file + ' in folder')
            
        if file!='':
            print('Loading file: ' + file)
            with h5py.File(folder + file,'r') as hdf5_file:
                for k in hdf5_file.keys(): 
                    #Note: Plenty to do here ....
                    #a=hdf5_file[k][()]
                    #if json 
                    #b=json.load(a)
                    print(k)

    def SaveToHDF5(self, ScanProject, override=1):
        """
        Saves the workspace to a file in the project folder with the name date_num_ScanProject.pkl'
        override:  if 0 makes a new file for each save (changing the num part of the file name) 
                   if 1 overides the last file, (finds the last num and overwrite)   
        """
        folder=ScanProject.Files['AnalysisConfig'][:-18]
        
        today=datetime.today().strftime('%Y-%m-%d')
        res=self.SheckFileExist(folder,'', startswith=today, endswith='_ScanProject.hdf5')    
        
        file= today + '_' + str(res) + '_ScanProject.hdf5'
        if override:
            if res!=0:
                file= today + '_' + str(res-1) + '_ScanProject.hdf5'
                print("OBS! Data found, will be overwitten")  
            
        with h5py.File(folder + file,'w') as hdf5_file:
            for i in ScanProject.__dict__: #dictionary of all data
                print(i)                   #print item name

                if isinstance(ScanProject.__dict__[i], dict): #sheck if item is a dictionary 
                    temp={}                                                  #tempory storige place of the item data 
                    for j in ScanProject.__dict__[i]:                        #go through the dictionary and   
                        if isinstance(ScanProject.__dict__[i][j],np.ndarray):#sheck if it is a numpy array
                            temp[j]= ScanProject.__dict__[i][j].tolist()        #convert array to list 
                        else:
                            temp[j]= ScanProject.__dict__[i][j]                 #do nothing                    
                    dset = hdf5_file.create_dataset(i, data=json.dumps(temp))#save the data with the name of the dictionary 

                elif isinstance(ScanProject.__dict__[i], list):
                    dset = hdf5_file.create_dataset(i, data=json.dumps(ScanProject.__dict__[i]))
                else:
                    dset = hdf5_file.create_dataset(i, data=ScanProject.__dict__[i])

            print("Data is hdf5 saved")

    def saveTrajtoMat(ScanProject, Dictionery,FName):    
        
        Trajectories={}
        for TrajToGet in range(len(Dictionery)):
            Data  = Dictionery[TrajToGet]
            name='T'+ str(TrajToGet)
            Trajectories[name]=Data
        
        folder=ScanProject.Files['AnalysisConfig'][:-18]
        io.savemat(folder + FName + '.mat', Trajectories)

    def get_cellMask(ScanProject):

        try: 
            ScanProject.CamCellMask*1.0
        except:
            path=ScanProject.Files['AnalysisConfig'][:-18] + 'widefield/'
            file='Mask.tif'

            print('No cell mask found, loading one from:')
            print('Try to load: ' + path + file)
            try:
                dataset = Image.open(path + file)
                h,w = np.shape(dataset)
                ScanProject.CamCellMask = np.zeros((h,w,dataset.n_frames))
                for i in range(dataset.n_frames):
                    dataset.seek(i)
                    ScanProject.CamCellMask[:,:,i] = np.array(dataset)       
            except:
                print('No cell mask found in folder, loading dummy mask')
                ScanProject.CamCellMask=np.ones([ScanProject.SetupSpecifics['NM'][0],ScanProject.SetupSpecifics['NM'][1],1])
                
        return ScanProject

    def load_MatFile(path,file):
        SData=io.loadmat(path+file)
        return SData



