from ConfigObject import ConfigObject # for .ini files
import numpy as np

class DataObject:
    
    def __init__(self, data_path_handler, FILE_TO_OPEN, loadSaveData=0):
        """
            Initiates a DataObject and defines parameters
            These are the default parameters assigned 2019-08-15
        """
        # Data files
        self.FILE_TO_OPEN=FILE_TO_OPEN
        self.loadSaveData=loadSaveData  #Is overrided to 1 if simulation, 
        self.Files=data_path_handler.extract_project_file_paths(FILE_TO_OPEN)
        self.ScanMatrix = np.array([])
        self.ScanInfo = []

        self.BasicStat = {}
        self.BasicStat['AvgTotalPhotons'] = []
        self.BasicStat['AvgTotalPhotonRate'] = []
        self.BasicStat['ThresholdPhotonRate'] = []
        self.BasicStat['AvgPolTotalPhotons'] = []
        self.BasicStat['AvgPolAboveBkg'] = []

        # Vector indicating if data point in ScanMatrix has passed the criteria
        self.filter_vector=[]

        # Trajectory trimming is now in the ini file
        #self.StartTrim=0.001           #[s]
        #self.EndTrim=0.001            #[s]
        #self.ForceMaxTrajLength=-1  #[s] -1 turned off, this needs to be worked on to get the right thing when it cuts
        #self.SavePlot=0

        ########################################################################################################################
        ############# Default Parameters the Shadowfax system    ###############################################################
        ########################################################################################################################               
        
        Project_config = ConfigObject(self.Files['AnalysisConfig'])
        self.PosCalibration = Project_config.PosCalibration
        
        ########################################################################################################################
        #############   Default Parameter for trajectory building and plotting    ###############################################
        ########################################################################################################################
        self.DataSelectionRules = Project_config.DataSelectionRules   
        self.TrajectorySelectionRules = Project_config.TrajectorySelectionRules  
        self.RegistrationCamToScan    = Project_config.RegistrationCamToScan  
        

        ########################################################################################################################
        #############   Default Parameter for microscopy Setup            ######################################################
        ########################################################################################################################

        # This should be moved either to the ini file for the phase retrieval or to normal ini file
        # May be better to have it in the normal ini file since these are setup specific 
        self.SetupSpecifics = Project_config.SetupSpecifics  
        
        # Remove these when cleaning out PFS_Parameters dependencies
        self.PFS_Parameters={}
        s=(0.822/2)*(self.SetupSpecifics['Lambda']/self.SetupSpecifics['Na'])*1e6  #[um] theoretical PSF width, the radius of the Airy disk is scaled to 1.22/2 and for the e-2 intensity drop that is std value one use 0.822/2
        self.PFS_Parameters['sigma'] =[s,s,2*s]             #[um] [x y z]
        
        ########################################################################################################################
        #############   Default Parameter for numerical PSF               ######################################################
        ########################################################################################################################
        self.Ph_Retrieval = Project_config.Ph_Retrieval
        ########################################################################################################################
        #############   Default Parameter for zCam                        ######################################################
        ########################################################################################################################
        self.zCam = Project_config.zCam
        ########################################################################################################################
        #############   Default Parameter for simulation                  ######################################################
        ########################################################################################################################
        is_simulation= self.SetupSpecifics['is_simulation']
        if is_simulation:
            self.loadSaveData=1
            self.Sim_par = Project_config.Sim_par

