from os import listdir
from os.path import isfile, join

class DataPathHandler:
    def __init__(self, prefix, pathnames):
        """
            Initiates a DataObject and defines parameters
            These are the default parmeters asigned 2019-08-15
        """
        # Data files paths to be build
        self.Files = {}
        self.Files['ScanData'] = []
        self.Files['PhotonTiming'] = []
        self.Files['ParameterInfo'] = []
        self.Files['AnalysisConfig'] = []

        self.prefix=prefix
        self.pathnames=pathnames
        
        self.full_path_names = [prefix + p for p in pathnames]


    def get_data_paths(self):
        """ 
       
        INFO:
            Inserts path information in to the DataObject.
            The data is stored as a dictionary of lists with the name "Files" and contains:
            Files['ScanData']
            Files['PhotonTiming']
            Files['ParameterInfo']
            Files['AnalysisConfig']

        INPUT : self.pathnames is a list of shadowfax data, normally in the folder structure defined by the following hierarchy:  date, experiment, ScanData.  Ex.: /20191211/chromosome_1986-1_19.5uW_500us/ScanData
        OUTPUT: With in the structure we get:
                1.  .Files['ScanData']  file pathes to the ScanData.bin file 
                2.  .Files['PhotonTiming']  file pathes to the PhotonTiming.bin file 
                3.  .Files['ParameterInfo']  file pathes to the ParameterInfo.bin file
                4.  .Files['AnalysisConfig'] file pathes to the AnalysisConfig.ini file

        """
        pathnames = [p + 'ScanData/' for p in self.full_path_names]
        
        for i in range(len(pathnames)):
        
            # Retriving file names in paths
            fileNames = [f for f in listdir(pathnames[i]) if isfile(join(pathnames[i], f))]

            for FileName in fileNames:
                # Find data with ScanData
                if FileName.find('[ScanData]') > -1:
                    self.Files['ScanData'].append(pathnames[i]+FileName)
            
                # Find data with PhotonTiming
                if FileName.find('[PhotonTiming]') > -1:
                    self.Files['PhotonTiming'].append(pathnames[i]+FileName)
            
                # Find data with Parameter information
                if FileName.find('Parameters') > -1:
                    self.Files['ParameterInfo'].append(pathnames[i]+FileName)
                    
            
            # Analysis configuration file
            self.Files['AnalysisConfig'].append(self.full_path_names[i] + 'AnalysisConfig.ini')      

        #Test that all files are found
        for state in self.Files:
            if len(self.Files[state])!=len(pathnames):
                print('Files in folder are missing, can not find all ' + state + ' files, found ' + str(len(self.Files[state])) + ' out of ' + str(len(pathnames)))
                print('This might affect the result!')
                input("Press Enter to continue...")

    def ExtractFilePath(self, FileNumberToOpen=0, DataType='ScanData'):
        """
        INPUT: 
            a. FileNumberToOpen is integer and is the scan project to open which is the row position in the pathnames list
            b. DataType a string: 'ScanData', 'PhotonTiming', 'ParameterInfo' or 'AnalysisConfig'
        """

        PathRequested = ''
        if self.Files != {}:
            if FileNumberToOpen < len(self.Files[DataType]):
                PathRequested = self.Files[DataType][FileNumberToOpen]
            else:
                print("ERROR: FileNumberToOpen >= len(self.Files['Path']), for "  + DataType)
        else:
            print("File paths are missing.")
            
        return  PathRequested   


    def extract_project_file_paths(self, FileNumberToOpen):
        """
            Extracts all file paths necesery for a specific project
        """
        project_requested={}
        for datatype in ['ScanData', 'PhotonTiming', 'ParameterInfo', 'AnalysisConfig']:
            PathRequested=self.ExtractFilePath(FileNumberToOpen, datatype)
            project_requested[datatype]=PathRequested
        
        return project_requested    
        
 

##### Class test code ##################
def main():
    # Path prefix for local data storage
    prefix = "C:/Users/elias.amselem/Desktop/Shadowfax"

    # Define path where the data is stored
    pathnames = ['/20191211/chromosome_1986-1_19.5uW_500us/',
             '/20190703/Data/Bead_3D_(5_Spots)_100nm_FullSpeed_0.5um-pk2pk/',
	        ]
                
    data_path_handler= DataPathHandler(prefix,pathnames)
    
    data_path_handler.get_data_paths()

    project_requested=data_path_handler.extract_project_file_paths(0)
    print(project_requested)

    

if __name__ == '__main__':
    main()