import numpy as np

class Raw_data_filters:

    def raw_data_filter(DataSelectionRules, ScanMatrix,wndw=1):
        """
        Looks through ScanMatrix and ScanInfo to create flags "interesting" data
        according to some selection criteria. "Interesting"
        data answers yes (1) to ALL of the following criteria.

        Selection Criteria:
        1. Is the sum total of photons over the sum of the threshold?
        2. Does the sum total of photons strictly reside within the allowed counts?
        3. Does the x data strictly reside within the allowed bounds?
        4. Does the y data strictly reside within the allowed bounds?
        5. Does the time data strictly reside within the allowed bounds?

        INPUT: 1. DataSelectionRules uses: 
                    a.  DataSelectionRules['APDThreshold'] 2x1 matrix as  [ADP 1 lower threshold ,  APD 2 lower threshold ]
                    b.  DataSelectionRules['PhotonCounts.CountRange'] 2x1 matrix with ADP 1 + APD_2 [sum lower threshold ,  sum upper threshold ]
                    c.  DataSelectionRules['PositionTime.xBounds']    2x1 matrix with x bounds [lower threshold ,  upper threshold ]
                    d.  DataSelectionRules['PositionTime.yBounds']    2x1 matrix with y bounds [lower threshold ,  upper threshold ]
                    e.  DataSelectionRules['PositionTime.TBounds']    2x1 matrix with T bounds [lower threshold ,  upper threshold ]

               2. ScanMatrix numpy array with 22 columns and N rows, the columns used in the filter are 0 to 5 coresponding to a though e i point 1 above.
               
        OUTPUT: vector same lenght as the ScanMatrix with enteries 1/0 Pass/Fail
        
        """
        APDThreshold = DataSelectionRules['APDThreshold']
        CountRange = DataSelectionRules['APDCountRange']
        # ErrorBound = DataSelectionRules['PhotonCounts.ErrorBound']
        xBounds = DataSelectionRules['xBounds']
        yBounds = DataSelectionRules['yBounds']
        tBounds = DataSelectionRules['TBounds']
        
        TotalData = ScanMatrix[0:5,:]

        wndwS=1 #if 1 it is a moving average if equal to wndw it is a fix window average
        if wndw==1:
            N=np.convolve(np.ones(wndw),TotalData[3,:] + TotalData[4,:])
        else:
            N=np.convolve(np.ones(wndw),TotalData[3,:] + TotalData[4,:])[0:-wndw+1:wndwS]
        
        # Get indices of data that pass selection criteria
        PassedSelection = np.all(
            (np.logical_and(np.greater_equal(TotalData[3,:], APDThreshold[0]),np.greater_equal(TotalData[4,:], APDThreshold[1])),
             np.logical_and(np.greater_equal(N, CountRange[0]), np.less_equal(N, CountRange[1])),
             np.logical_and(np.greater(TotalData[1,:], xBounds[0]), np.less(TotalData[1,:], xBounds[1])),
             np.logical_and(np.greater(TotalData[2,:], yBounds[0]), np.less(TotalData[2,:], yBounds[1])),
             np.logical_and(np.greater(TotalData[0,:], tBounds[0]), np.less(TotalData[0,:], tBounds[1])))
            , axis=0)

        print('Filter vector calculated')

        return PassedSelection

    def scan_matrix_reduction(passed_selection, scan_matrix):
        
        return scan_matrix_reduced
