import matplotlib.pyplot as plt
import numpy as np



def ViscosityMass(T,waterMass,glycerolMass):
    """
    This python code is based on the MatLab code orginaly provided by Chris Westbrook
    http://www.met.reading.ac.uk/~sws04cdw/viscosity_calc.html
    """

    #Densities ----------------

    glycerolDen = (1273.3-0.6121*T)/1000 			#Density of Glycerol (g/cm3)
    waterDen = (1-np.power(((abs(T-4))/622),1.7)) 	#Density of water (g/cm3)


    #Fraction cacluator ----------------
    glycerolVol=glycerolMass/glycerolDen
    waterVol=waterMass/waterDen
    totalMass=glycerolMass+waterMass
    mass_fraction=glycerolMass/totalMass
    vol_fraction= glycerolVol/(glycerolVol+waterVol)

    print ("Mass fraction of mixture =", np.round(mass_fraction,decimals=5))
    print ("Volume fraction of mixture =", np.round(vol_fraction,decimals=5))


    #Density calculator ----------------

    ##Andreas Volk polynomial method
    contraction_av = 1-np.power(3.520E-8*((mass_fraction*100)),3)+np.power(1.027E-6*((mass_fraction*100)),2)+2.5E-4*(mass_fraction*100)-1.691E-4
    contraction = 1+contraction_av/100

    ## Distorted sine approximation method
    #contraction_pc = 1.1*math.pow(math.sin(numpy.radians(math.pow(mass_fraction,1.3)*180)),0.85)
    #contraction = 1 + contraction_pc/100

    density_mix=(glycerolDen*vol_fraction+waterDen*(1-vol_fraction))*contraction

    print ("Density of mixture =",np.round(density_mix,decimals=5),"g/cm3")


    #Viscosity calcualtor ----------------

    glycerolVisc=0.001*12100*np.exp((-1233+T)*T/(9900+70*T))
    waterVisc=0.001*1.790*np.exp((-1230-T)*T/(36100+360*T))

    a=0.705-0.0017*T
    b=(4.9+0.036*T)*np.power(a,2.5)
    alpha=1-mass_fraction+(a*b*mass_fraction*(1-mass_fraction))/(a*mass_fraction+b*(1-mass_fraction))
    A=np.log(waterVisc/glycerolVisc)

    viscosity_mix=glycerolVisc*np.exp(A*alpha)

    print ("Viscosity of mxiture =",np.round(viscosity_mix,decimals=5), "Ns/m2")
    return viscosity_mix

def get_D(T,Rh,viscosity_mix):
    #Rh=1.7e-9         #Thermodynamic radius (nm)
    #T= 20             #Temperature in C
    #viscosity_mix     #viscosity (Ns/m2)=(kg/m·s)
    kb=1.38064852e-23 #m^2*kg/ (s^2 K) Boltzman constant
    Tk=T+273.15       #Temp in kelvin
    
    #D=1               #um2/s 
    D=(kb*Tk/(6*np.pi*viscosity_mix*Rh))*1e12 #um2/s
    return D


if __name__ == '__main__':
    
    T = 20 				#temperature (degrees Celcius)
    MWT=575.58          #Daltond for JF549 
    for n in range(5):
        Rh=(np.linspace(0.589,0.740,num=5)*1e-9)[n]         #Hydrodynamic radius (nm)    
        for i in range(2):
            if i==0:
                glycerolMass=np.array([85.94,88.15,90.65,92.46,94.11,95.81,98.08])
            else:
                glycerolMass = np.linspace(85,100,num=50) #Mass of Glycerol used (g)  

            waterMass = 100-glycerolMass 		      #Mass of water required (g)
            viscosity_mix=ViscosityMass(T,waterMass,glycerolMass) #viscosity of mixture (Ns/m2)=(kg/m·s)
            D=get_D(T,Rh,viscosity_mix)
            if i==0:
                plt.plot(glycerolMass,D,'*')
            else:
                plt.plot(glycerolMass,D,'-',label='Rh:'+str(np.round(Rh*1e9,decimals=2))+'nm')


    plt.grid(which='both')
    plt.xlabel('glycerolMass precent of tot mass')
    plt.ylabel('D [um2/s]')
    plt.legend()
    plt.show()


    