#!/usr/bin/env python3.6-64
from DataObject import DataObject # Object that will hold all relevent data
from PipelineOrganizer import PipelineOrganizer as _PO
from DataHandler import DataHandler as _DH
from DataPathHandler import DataPathHandler
from HMM import HMM_Traj as _HMM

import numpy as np
import seaborn as sn
import matplotlib.pyplot as plt
from PlotLibrary import TrajPlots as _TP
#from PlotLibrary import RawDataPlot as _RawDataPlot

def main():
    # Path prefix for local data storage
    # prefix = "/proj/uppstore2018129/elflab/Projects/Shadowfax/Data"
    # prefix = "C:/Users/bobro/Documents/ShadowDev/Data"
    prefix = "C:/Users/elias/Desktop/Shadowfax/Data"
    
    # Define path where the data is stored  
    
    pathnames = ['/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-1/' ,  #0
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-2/' ,  #1
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-3/' ,  #2
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-4/' ,  #3
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-5/',   #4
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-6/',   #5
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-7/',   #6
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-8/',   #7
                 '/20210319_PSF_CellTracking_TriggerFactor/data/240mW_trigger-factor-9/',   #8
                ]

    #Setup the data path handler and DataHandler
    data_path_handler= DataPathHandler(prefix,pathnames)
    data_path_handler.get_data_paths() # pathes are stored in, data_path_handler.Files
    DataHandler=_DH()  # Class object that can process data 

    ScanProject_All={} # Dictionary for all projects
    for Project_Folder in range(9):
        ScanProject_All[Project_Folder] = DataObject(data_path_handler, Project_Folder,loadSaveData=1) #Class object that stores all data for a given scan
        ScanProject_All[Project_Folder] =_PO.DataPipline(ScanProject_All[Project_Folder],DataHandler) 
            
        print("Loading project number: " + str(Project_Folder))
    print("Loading is done!")

    Dtxyz={}
    for Project_Number in ScanProject_All:
        ScanProject=ScanProject_All[Project_Number]
        ScanProject.TrajectorySelectionRules['MinTrajectoryTimeLength']=0.05 #[s]
        PhotonThreachhold=8
        wndw=10
        UseCellMask=True
        MovieStepSize=0
        ScanProject.PoissonProbXYZMap_NumPFSFiltred=DataHandler.get_traj_toImagCoordinates(ScanProject, ScanProject.PoissonProbXYZMap_NumPFS)
        ScanProject.PoissonProbXYZMap_NumPFSCellMaskTrajFlag=DataHandler.get_traj_inCells(ScanProject, ScanProject.PoissonProbXYZMap_NumPFSFiltred,mask=UseCellMask)
        ScanProject.PoissonProbXYZMap_NumPFSFiltred=DataHandler.get_traj_filter(ScanProject, ScanProject.PoissonProbXYZMap_NumPFSFiltred, ScanProject.PoissonProbXYZMap_NumPFSCellMaskTrajFlag,PhotonThreachhold=PhotonThreachhold,wndw=wndw)
        #_TP.plot_ovelayTrajOnCells2(ScanProject,ScanProject.PoissonProbXYZMap_NumPFSFiltred,CellMask=True,do_save_plot=1)

        DataHandler.get_MSD_traj(ScanProject, ScanProject.PoissonProbXYZMap_NumPFSFiltred)

            #CVE D and sigma evaluation
        for TrajToGet in  range(int(len(ScanProject.txyzCVEMSD_traj)/3)):
            txyzCVE_D=ScanProject.txyzCVEMSD_traj[str(TrajToGet),'D']*1.0 #[mu^2/s]
            txyzCVE_D[:,0]=(txyzCVE_D[:,0]-txyzCVE_D[0,0])*1e3 #[ms]
            Dtxyz[Project_Number,TrajToGet]=txyzCVE_D

    print('D are calculated')
    ######################################################################
    #Find HMM
    DBias=100
    HMM_n_components=3
    Dinv_all=()
    for index in Dtxyz:
        Dinv=1/(np.mean(Dtxyz[index][:,1:],axis=1)+DBias)
        Dinv_all+=(Dinv,)
    dt_est=np.mean(np.diff(Dtxyz[0,0][:,0])) #ms
    

    FoundModelDic=_HMM.train_HMM(X=Dinv_all, n_components=HMM_n_components,BootStrap=100)
    predicted_state_sequence=_HMM.predict_sequence(model=FoundModelDic[0],observed_sequence=Dinv_all)
    HMM_Models=OrgenizeHMM_parm(FoundModelDic,dt_est,DBias)

    #_HMM.plot_hist_sequence_with_fit2(Dinv_all,HMM_Model,xoffset=DBias)

    ## Predict 
    index=0
    HMM_prediction=()
    HMM_DistVariancePrediction=()
    for Project_Number in ScanProject_All:
        ScanProject=ScanProject_All[Project_Number]
        for TrajToGet in range(len(ScanProject.PoissonProbXYZMap_NumPFSFiltred)):
            HMM_prediction+= (np.array(HMM_Model['D'])[predicted_state_sequence[index]],)
            HMM_DistVariancePrediction+= (np.array(HMM_Model['VarinvGamma'])[predicted_state_sequence[index]],)

            #txyzCVE_D=np.mean(ScanProject.txyzCVEMSD_traj[str(TrajToGet),'D'][:,1:]*1.0,axis=1) #[mu^2/s]
            #Xi=np.sum(((HMM_prediction[index]-txyzCVE_D)**2)/HMM_DistVariancePrediction)
            index+=1



    ### Plot
    index=0
    for Project_Number in ScanProject_All:
        ScanProject=ScanProject_All[Project_Number]
        for TrajToGet in range(len(ScanProject.PoissonProbXYZMap_NumPFSFiltred)):
            MovieStepSize=0
            _TP.plot_Hist_Traj_MDS7(ScanProject,TrajToGet,ScanProject.PoissonProbXYZMap_NumPFSFiltred,HMM_prediction[index],HMM_Model,MovieStepSize=MovieStepSize,do_save_plot=0)
            index+=1


    print('end')
    #for TrajToGet in range(len(ScanProject.PoissonProbXYZMap_NumPFSFiltred)):
    #    _TP.plot_Hist_Traj_MDS6(ScanProject,TrajToGet,ScanProject.PoissonProbXYZMap_NumPFSFiltred,MovieStepSize=MovieStepSize,do_save_plot=1)


def OrgenizeHMM_parm(FoundModelDic,dt_est,DBias):
    """
    The relation between D "mean", M "Mode" of the inv-Gamma and alpha and beta of the distribution is
    alpha=(M+D)/(M-D) or D=beta/(alpha-1)
    beta=2*D*M/(M-D)  or M=beta/(alpha+1)
    """
    HMM_Models={}
    for ModelIndex in  FoundModelDic:
        FoundModel= FoundModelDic[ModelIndex]
        new_trans_mat, dist_params =_HMM.get_model_parm(FoundModel) 

        D_estimated=()
        for state in dist_params:
            D_estimated+=(dist_params[state][1]/(dist_params[state][0]-1) -DBias,) #beta/(alpha-1) invers-Gamma mean

        M_estimated=()
        for state in dist_params:
            M_estimated+=(dist_params[state][1]/(dist_params[state][0]+1) -DBias,) #beta/(alpha+1) invers-Gamma mode

        Dwelltime_estimated=()
        for i in range(new_trans_mat.shape[0]):
            Dwelltime_estimated+=(dt_est/(1-new_trans_mat[i,i]),) #[ms]
        Dwelltime_estimated=np.array(Dwelltime_estimated)
    
        Occupancy_estimated=()
        for Dwellt in Dwelltime_estimated: 
            Occupancy_estimated+=(Dwellt/np.sum(Dwelltime_estimated),)
        Occupancy_estimated=np.array(Occupancy_estimated)

        VarinvGamma=()
        for state in dist_params:
            param=dist_params[state]
            VarinvGamma+=(param[1]**2/((param[0]-2)*(param[0]-1)**2),)


        HMM_Model={}
        HMM_Model['TransMat']=new_trans_mat
        HMM_Model['Distributions']=dist_params
        HMM_Model['D']=D_estimated
        HMM_Model['M']=M_estimated
        HMM_Model['VarinvGamma']=VarinvGamma
        HMM_Model['Dwelltime']=Dwelltime_estimated
        HMM_Model['Occupancy']=Occupancy_estimated
        HMM_Model['DBias']=DBias

        HMM_Models[ModelIndex]=HMM_Model
    
    get_HMMCovMatrix(HMM_Models)
    
    return HMM_Models

def get_HMMCovMatrix(HMM_Models_In):
    """
    This produces the covariance matrix of the transition matrix together with the diffusion estimation and mode estimation 
    the NxN transition matrix is reduced by removing the diagonal (since each row is normolized to 1) and each row is extended 
    to include D and M thus the new matrix is (N-1+2)xN.
    this is then flattend per row thus the element order for a 3 state system is 
    Xi = [a12 a13 D1 M1 a21 a23 D2 M2 a31 a32 D3 M3]
    
    this vektor Xi contains the diffent estiamtions from original data (i=0) and bootstrap (i>0)
    from these vectors we construct the cocvariance matrix elemets which are 
    C(k,l)=E[( Xi(k)-E[Xi(k)]_i )( Xi(l)-E(Xi(l))_i )]_i
    Where the mean is taken over the index i which is the bootstrap data
    
    The HMM fit do not have an order in its parameters, thus the bootstrap HMM models are not alined to each other. The order we will use here is
    States will be orderd according to the rules:
        1. Diffuision konstantes are orderd from small to big 


    """
    
    print('Getting covariance matrix')
    HMM_Models=HMM_Models_In.copy()

    #Order all models to each other 
    DStateRank0=np.argsort(np.argsort(HMM_Models[0]['D']))
    TStateRank0=np.argsort(np.argsort(HMM_Models[0]['Dwelltime']))
    #print(HMM_Models[0]['D'])
    #print(DStateRank0)
    #print('##################')
    if len(HMM_Models)>1:
        for index in range(1,len(HMM_Models)):
            #print(HMM_Models[index]['D'])
            HMM_Model=HMM_Models[index].copy()
            DStateRank=np.argsort(np.argsort(np.array(HMM_Model['D'])))
            TStateRank=np.argsort(np.argsort(HMM_Model['Dwelltime']))
            Dmap=()
            Tmap=()
            for i in range(len(DStateRank0)):
                Dmap+=(np.where(DStateRank0[i]==DStateRank)[0][0],)
                #Tmap+=(np.where(TStateRank0[i]==TStateRank)[0][0],)
            Smap=Dmap
            #print('rank:' + str(DStateRank))
            #print('map  :' + str(Smap))

            new_trans_mat=np.zeros(HMM_Model['TransMat'].shape)
            for i in range(len(DStateRank0)):
                for j in range(len(DStateRank0)):
                    new_trans_mat[Smap[i],Smap[j]]=HMM_Model['TransMat'][i,j]
            HMM_Model['TransMat']=new_trans_mat

            new_Distributions={}
            for i in range(len(HMM_Model['Distributions'])):
                new_Distributions['s' + str(i)]=HMM_Model['Distributions']['s' + str(Smap[i])]
            HMM_Model['Distributions']=new_Distributions

            HMM_Model['D']=tuple(np.array(HMM_Model['D'])[np.array(Smap)])
            HMM_Model['M']=tuple(np.array(HMM_Model['M'])[np.array(Smap)])
            HMM_Model['VarinvGamma']=tuple(np.array(HMM_Model['VarinvGamma'])[np.array(Smap)])
            HMM_Model['Dwelltime']=tuple(np.array(HMM_Model['Dwelltime'])[np.array(Smap)])
            HMM_Model['Occupancy']=tuple(np.array(HMM_Model['Occupancy'])[np.array(Smap)])
            HMM_Model['DBias']=HMM_Model['DBias']

            HMM_Models[index]=HMM_Model
            #print(HMM_Models[index]['D'])
            #print('-------next------------')

        X=np.zeros((len(HMM_Models),len(HMM_Models[0]['D'])**2 + 2*len(HMM_Models[0]['D'])-len(HMM_Models[0]['D'])))
        for index in range(len(HMM_Models)):
            HMM_Model=HMM_Models[index].copy()
            #Creating the relevent vector that we will construct the covariance
            ReducedTransMat=HMM_Model['TransMat'][~np.eye(HMM_Model['TransMat'].shape[0],dtype=bool)].reshape(HMM_Model['TransMat'].shape[0],-1)
            X[index,:]=(np.vstack((ReducedTransMat.T,HMM_Model['D'],HMM_Model['M'])).T).flatten()
        CovMatrix=np.cov(X.T,bias=False)
        x_axis_labels=['a12', 'a13', 'D1', 'M1', 'a21', 'a23', 'D2', 'M2', 'a31', 'a32', 'D3', 'M3']
        sn.heatmap(np.round(CovMatrix,decimals=4), annot=True, fmt='g',xticklabels=x_axis_labels, yticklabels=x_axis_labels)
        plt.show()
        print('Done!')

if __name__ == '__main__':
    main()




    """
    


        for state in dist_params:
            S=StateOrder[index]
            D_estimatedRe+=(D_estimated[S],)
            Dwelltime_estimatedRe+=(Dwelltime_estimated[S],)
            VarinvGammaRe+=(VarinvGamma[S],)
            Occupancy_estimatedRe+=(Occupancy_estimated[S],)
            dist_paramsRe['s' + str(index)]=dist_params['s'+str(S)]
            new_trans_matRe[:,index]=new_trans_mat[:,S]
    """