#!/usr/bin/env python3.6-64
from DataObject import DataObject # Object that will hold all relevent data
from PipelineOrganizer import PipelineOrganizer as _PO
from DataHandler import DataHandler as _DH
from DataPathHandler import DataPathHandler
from PlotLibrary import TrajPlots as _TP


def DataPipline(ScanProject,DataHandler,loadSaveData=1):
    loadSaveData=ScanProject.loadSaveData
    if loadSaveData:
        ScanProject1 = DataHandler.UnPickleData(ScanProject)
        ScanProject1.Files['AnalysisConfig']=ScanProject.Files['AnalysisConfig']
        ScanProject=ScanProject1
        DataHandler.get_ini_file(ScanProject) #Reload ini file if changes has ocurred 

        if ScanProject.SetupSpecifics['is_simulation']:
            DataHandler.CalcBasicPhotonStats(ScanProject) #Calculations are stored in, ScanObject.BasicStat

            DataHandler.get_raw_data_filter(ScanProject)  #Data stored in, ScanObject.filter_vector
            DataHandler.get_TrajObject() #Initiate trajectory function to the DataHandler
            DataHandler.get_Trajectories(ScanProject) # Calculations are stored in, TrajObject.Trajs, TrajObject.TrajInfo
                
            print('Amount of trajectoris found:' + str(ScanProject.Trajs.shape[1]) )

            DataHandler.PickleData(ScanProject,override=0)
    else: #Run the trajectory estimations
        DataHandler.getScanMatrix(ScanProject)   #Data stored in, ScanObject.ScanMatrix, ScanObject.ScanInfo
        DataHandler.CalcBasicPhotonStats(ScanProject) #Need to go through calculation and see what it is doing (Do not trust), Calculations are stored in, ScanObject.BasicStat
            
        DataHandler.get_raw_data_filter(ScanProject)  #Data stored in, ScanObject.filter_vector
        DataHandler.get_TrajObject() #Initiate trajectory function to the DataHandler
        DataHandler.get_Trajectories(ScanProject) # Calculations are stored in, TrajObject.Trajs, TrajObject.TrajInfo

        DataHandler.do_TrimTrajs(ScanProject, ScanProject.StartTrim, ScanProject.EndTrim, ScanProject.ForceMaxTrajLength)
        print('Amount of trajectoris found:' + str(ScanProject.Trajs.shape[1]) )
    
        if 0: #Plot photon counts for each traj
            #AmountOfTraj=len(ScanProject.PoissonProbXYZMap_NumPFS)
            AmountOfTraj=ScanProject.Trajs.shape[1]
            for TrajToGet in range(AmountOfTraj): 
                _TP.plotPhotonCount(ScanProject,TrajToGet,do_save_plot=1)
            print('plotPhotonCount function Done!')
        
        # Trajectory estimations  
        if 1: # Gaussian
            DataHandler.get_Gaussian_traj_estimation(ScanProject) # Data saved under ScanProject.TrajPosEstimation
            print("Gaussian traje Done!")

        if 1: # Poisson Map with numerical PSF and fix amount of laser shots
              # After each trajectory the structure will be pickeled and saved  
            #TrajToGet=1
            #DataHandler.get_traj_Poisson_ProbMap_NumPFS(ScanProject,TrajToGet,verbose=0, do_onlinesave=0) # Calculations are stored in, ScanProject.TrajObject.PoissonProbXYMap
            DataHandler.get_All_traj_Poisson_ProbMap_NumPFS(ScanProject)                    
            print("Poisson Map with numerical PSF traje Done!")
            
        if 1: #Pickel data                   
            DataHandler.PickleData(ScanProject,override=0)
            #DataHandler.savetoMat(ScanProject) #Change to correct saveTrajtomat

    if 1: # Calculate traj mean and std and plot it
        AmountOfTraj=len(ScanProject.PoissonProbXYZMap_NumPFS)
        for TrajToGet in range(AmountOfTraj): 
            #Plot is saved as name: T_1_M_STD_traj.png
            _TP.plot_Hist_Traj_Mean_STD(ScanProject,TrajToGet,wndw=100,do_save_plot=1)
            
    if 0: #Plot Piezo stage for each traj
        AmountOfTraj=len(ScanProject.PoissonProbXYZMap_NumPFS)
        for TrajToGet in range(AmountOfTraj): 
            _TP.plotPiezostages(ScanProject,TrajToGet,do_save_plot=0)
                
    return ScanProject

def main():
    # Path prefix for local data storage
    prefix = "C:/Users/elias/Desktop/Shadowfax/Data"
    
    # Define path where the data is stored  
    # In each project foler one need to have:
    # 1. ssystem ini 
    # 2. under the folder ScanData in the project folder one need to have the files
    #    a. Parameters.xml
    #    b. "something"[PhotonTiming].bin
    #    c. "something"[ScanData].bin

    pathnames = ['/20210114_PSF_Bead_rotation_with_weights/Bead_20nm_SpinAgarose_102_98_Rotating_xy0.3_zp2p0.33/',#0
                ]
 
    for Project_Folder in [0]:
        loadSaveData=0  #if 1 load saved (pickeled) data, if 0 do not load (re-calculate)

        data_path_handler= DataPathHandler(prefix,pathnames)
        data_path_handler.get_data_paths() # pathes are stored in, data_path_handler.Files
    
        DataHandler=_DH() #Class object that can process data 
        ScanProject = DataObject(data_path_handler, Project_Folder,loadSaveData) #Class object that stores all data for a given scan
        ScanProject = DataPipline(ScanProject,DataHandler) 
            
        print("Done!")


if __name__ == '__main__':
    main()

