from DataObject import DataObject # Object that will hold all relevent data
from PipelineOrganizer import PipelineOrganizer as _PO
from DataHandler import DataHandler as _DH
from DataPathHandler import DataPathHandler
from PSF_Lib import PSF_Lib as _PSF
from Physics_Lib import Particle as _Particle
from Physics_Lib import physics_Lib as _PhLib
from PlotLibrary import TrajPlots as _TP
from Simulator import Simulator as _Sim
from miscellaneous import miscellaneous as _MI
from HMM import HMM_Traj as _HMM

import scipy.io as io
import numpy as np
import scipy.stats as stats    

from mpl_toolkits.mplot3d import Axes3D
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.cm as cm
import mpl_toolkits.mplot3d.axes3d as p3
from matplotlib.gridspec import GridSpec
import scipy as sp
import scipy.interpolate
from scipy.interpolate import RegularGridInterpolator

"""
This file will run the main_2() function which will output figures in the project root folder two steps up from this folder.
    CVE_w16_j1_n1_F0.svg                Plots of the CVE method
    ECVE_lineFitt_w16_j16_n1_F2.svg     Spesial tests including a linear fit, not used
    ECVE_lineFitt0_w16_j16_n1_F2.svg    Spesial tests including a linear fit, not used  
    ECVE_w16_j8_n1_F2.svg               Plot  of the ECVE 
    MSD_w16_j16_n1_F2.svg               Plots of the MSD method

We generate 20 periods of 200 + 200 + 200 points each with different diffusion coeficient (the sampling is done by 10x sub-sampling and then we average over 10 points to include motion blur)

"""


def main_1():
    """
    Test of MSD and CVE for a simulated Brownian motion
    This is a addaptation of the test done in MethodAnalysis produced in 2021
    This is an older version use the main_2 function as it is an uppdated version. 

    """
    do_save_plot=1
    # Path prefix for local data storage
    #prefix target folder:  20220519_(SimulationDiffusionTest)
    path = "./.."


    #0:CVE        Dwndw>1; j=1;n=1;Flag=0
    #1: ECVE      Dwndw>2; j>2;n=1;Flag=2 Current: Dwndw=j
    #2: ECVE line fitt     j>2; Dwndw=1; n=1;Flag=2
    #3:ECVE line fitt v0   j>2; Dwndw=1; n=1;Flag=2
    #4: MSD

    Dwndw=10;
    MFlags= range(5)
    

    #Simulation track
    steps=8000
    substeps=10       #Simulate blur this will give R=1/6.2
    #T=1              #[s]
    dt=0.84e-3        #[s]
    sigma_e= 70e-3    #[um]
    sigma_e_z=sigma_e #[um]
    D_theory1=0.1     #[um^2/s]
    steps1=200
    D_theory2=5       #[um^2/s]
    steps2=200
    DBias=0.6         #For analysis 


    # Analysis 
    wndw=5 #not in use
    wndwS=1 #if 1 it is a moving average, if equal to wndw it is a fix window average
    #############################################################################
    # Create 3D Brownian motion 
    k=np.arange(steps)
    DTrue=np.array([])
    dxdydz=np.array([[0,0,0]])
    for i in range(np.ceil(steps/(steps1+steps2)).astype('int')):
        dxdydz1 = np.random.normal(0, np.sqrt(2*D_theory1*(dt/substeps)), (int(steps1*substeps), 3))
        dxdydz=np.concatenate((dxdydz,dxdydz1))
        dxdydz2 = np.random.normal(0, np.sqrt(2*D_theory2*(dt/substeps)), (int(steps2*substeps), 3))
        dxdydz=np.concatenate((dxdydz,dxdydz2))
        DTrue=np.concatenate((DTrue,np.ones(steps1)*D_theory1))
        DTrue=np.concatenate((DTrue,np.ones(steps2)*D_theory2))

            
    trajT = np.cumsum(dxdydz[:-1,:], axis=0)
    traj=np.zeros((steps,3))
    for i in range(3):
        if substeps>1:
            traj[:,i]= np.convolve(np.ones(substeps)/substeps,trajT[:,i])[substeps-1:-substeps+1:substeps]
        else:
            traj[:,i]=trajT[:,i]
    

    X=traj[:,0] + np.random.normal(0, sigma_e, steps)      #[um]
    Y=traj[:,1] + np.random.normal(0, sigma_e, steps)      #[um]
    Z=traj[:,2] + np.random.normal(0, sigma_e_z, steps)     #[um]
    t_track=k*dt 

    txyzn_Map=np.vstack((t_track,X,Y,Z,k))

    TrajDictionary={}
    TrajToGet=0
    TrajDictionary[TrajToGet]=txyzn_Map
    ################################################################################################
    ### Estimate D and Sigma
    ################################################################################################
    for MFlag in MFlags:
        #CVE estimation of D and sigma
        if MFlag==0:
            j=1;n=1;Flag=0
            result_txyz=_PhLib.get_CVE_MSD_traj(TrajDictionary,wndw=Dwndw,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='CVE'
        elif MFlag==1:
            j=Dwndw;n=1;Flag=2
            result_txyz=_PhLib.get_CVE_MSD_traj(TrajDictionary,wndw=Dwndw,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='ECVE'
        elif MFlag==2:
            j=Dwndw;n=1;Flag=2
            result_txyz=_PhLib.get_CVE_traj_Special(TrajDictionary,wndw=1,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='ECVE_lineFitt'
        elif MFlag==3:
            j=Dwndw;n=1;Flag=2
            result_txyz=_PhLib.get_CVE_traj_Specialv0(TrajDictionary,wndw=1,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='ECVE_lineFitt0'
        elif MFlag==4:
            result_txyz=_PhLib.get_MSD_traj2(ScanProject=None,TrajDictionary=TrajDictionary,window=Dwndw,MSDStart=1,MSDSteps=Dwndw-1,PointsToUse=Dwndw-1,R=1/6.2,Flag=0)
            Name_prefix='MSD'

        t=result_txyz[str(TrajToGet),'D'][:,0]
        D=np.transpose(result_txyz[str(TrajToGet),'D'][:,1:])
        S2=np.transpose(result_txyz[str(TrajToGet),'Sigma2'][:,1:])
        #VarD=np.transpose(result_txyz[str(TrajToGet),'VarD'][:,1:])
    
        Name_Mid='_w' + str(Dwndw) + '_j'+ str(j) + '_n' + str(n) + '_F' + str(Flag)


            #Create runing mean 
        MD=np.mean(D,axis=0)
        MD = np.convolve(np.ones(wndw)/wndw,MD)
        MD=MD[wndw-1:-wndw+1:wndwS]

        MS2 = np.mean(S2,axis=0)
        MS2 = np.convolve(np.ones(wndw)/wndw,MS2)
        MS2 = MS2[wndw-1:-wndw+1:wndwS]

        Mt = np.convolve(np.ones(wndw)/wndw,t)
        Mt=Mt[wndw-1:-wndw+1:wndwS]

        DTrueTrunc=[]
        for indextime in range(len(t)):
            DTrueTrunc=np.concatenate((DTrueTrunc,[DTrue[np.argmin((t_track-t[indextime])**2)]]))
            
        DTrueTrunc1_Pointer=np.where(DTrueTrunc==D_theory1)
        DTrueTrunc2_Pointer=np.where(DTrueTrunc==D_theory2)
    
        #Fitt Gamma function 
        fit_alpha, fit_loc, fit_scale=stats.gamma.fit(np.mean(D+DBias,axis=0))
        invfit_alpha, invfit_loc, invfit_scale=stats.invgamma.fit(np.mean(D+DBias,axis=0))
        invfit_alpha2, invfit_loc2, invfit_scale2=stats.invgamma.fit(np.mean(D,axis=0))

        DFittParam=np.linspace(0,30,num=1000)
        PDF_Gamma=stats.gamma.pdf(DFittParam, a=fit_alpha,loc=fit_loc, scale=fit_scale)
        PDF_invGamma=stats.invgamma.pdf(DFittParam, a=invfit_alpha,loc=invfit_loc, scale=invfit_scale)
        PDF_invGamma2=stats.invgamma.pdf(DFittParam, a=invfit_alpha2,loc=invfit_loc2, scale=invfit_scale2)

        DFittParam=DFittParam-DBias
        fit_beta=1/fit_scale
        invfit_beta=invfit_scale 
        invfit_beta2=invfit_scale2 
        GammaMean=(fit_alpha/fit_beta)+fit_loc-DBias
        GammaMode=((fit_alpha-1)/fit_beta)+fit_loc-DBias
        invGammaMean=(invfit_beta/(invfit_alpha-1))+invfit_loc-DBias
        invGammaMode=(invfit_beta/(invfit_alpha+1))+invfit_loc-DBias
    
        invGammaMean2=(invfit_beta2/(invfit_alpha2-1))+invfit_loc2
        invGammaMode2=(invfit_beta2/(invfit_alpha2+1))+invfit_loc2

        ###############################################################
        # Figure construction CVE
        fig = plt.figure(constrained_layout=True)
        fig.set_size_inches(2*6, 2*5, forward=True)
        gs = GridSpec(3,4, figure=fig)
        ax0=fig.add_subplot(gs[0, 0:4]) #Trajectory 
        ax1=fig.add_subplot(gs[1, 0:4]) #Diffusion
        ax2=fig.add_subplot(gs[2, 0:4]) #Localization error 
        #ax3=fig.add_subplot(gs[1, 3]) # Diffusion histogram 
        #ax4=fig.add_subplot(gs[2, 3]) #Localization error  histogram

        ax0.set_title(Name_prefix + ' method for D and sigma')
        for i in range(np.floor(steps/(steps1+steps2)).astype('int')):
            if i==0:
                ax0.axvspan(txyzn_Map[0,i*(steps1+steps2)], txyzn_Map[0,i*(steps1+steps2)+steps1], alpha=0.5, color='white', label='D:'+str(D_theory1)+ ' t:' + str(steps1*dt))
                ax0.axvspan(txyzn_Map[0,i*(steps1+steps2)+steps1], txyzn_Map[0,(i+1)*(steps1+steps2)-1], alpha=0.2, color='gray', label='D:'+str(D_theory2)+ ' t:' + str(steps2*dt))
            else:
                ax0.axvspan(txyzn_Map[0,i*(steps1+steps2)], txyzn_Map[0,i*(steps1+steps2)+steps1], alpha=0.5, color='white')
                ax0.axvspan(txyzn_Map[0,i*(steps1+steps2)+steps1], txyzn_Map[0,(i+1)*(steps1+steps2)-1], alpha=0.2, color='gray')
        ax0.plot(txyzn_Map[0,:],txyzn_Map[1,:],'r'); ax0.plot(txyzn_Map[0,:],txyzn_Map[2,:],'g'); ax0.plot(txyzn_Map[0,:],txyzn_Map[3,:],'b')
        ax0.set_ylabel('um');ax0.grid();ax0.legend()

            
        # Diffusion constant estimation 
        #ax1.plot(t,D[0,:],'r'); ax1.plot(t,D[1,:],'g'); ax1.plot(t,D[2,:],'b')
        for i in range(np.floor(steps/(steps1+steps2)).astype('int')):
            #ax1.axvspan(txyzn_Map[0,i*(steps1+steps2)], txyzn_Map[0,i*(steps1+steps2)+steps1], alpha=0.5, color='white')
            ax1.axvspan(txyzn_Map[0,i*(steps1+steps2)+steps1], txyzn_Map[0,(i+1)*(steps1+steps2)-1], alpha=0.2, color='gray')
        ax1.plot(t,np.mean(D,axis=0),'k')
        #ax1.plot(t_MSD,np.mean(D_MSD,axis=0),'m')
        #ax1.plot(Mt,MD,'m') 
        ax1.plot(t_track,DTrue,'r') 
        #ax1.plot(Mt,np.ones(len(Mt))*D_theory1,'r') 
        #ax1.plot(Mt,np.ones(len(Mt))*D_theory2,'r')
        ax1.set_ylabel('um^2/s');ax1.grid()
        if MFlag!=2:
            #Localization estimation 
            #ax2.plot(t,S2[0,:],'r'); ax2.plot(t,S2[1,:],'g'); ax2.plot(t,S2[2,:],'b')
            for i in range(np.floor(steps/(steps1+steps2)).astype('int')):
                #ax1.axvspan(txyzn_Map[0,i*(steps1+steps2)], txyzn_Map[0,i*(steps1+steps2)+steps1], alpha=0.5, color='white')
                ax2.axvspan(txyzn_Map[0,i*(steps1+steps2)+steps1], txyzn_Map[0,(i+1)*(steps1+steps2)-1], alpha=0.2, color='gray')        
            ax2.plot(t,np.mean(S2,axis=0)*1e6,'k')
            #ax2.plot(t_MSD,np.mean(S2_MSD,axis=0)*1e6,'m')
            #ax2.plot(Mt,MS2*1e6,'m')
            ax2.plot(Mt,(np.ones(len(Mt))*sigma_e**2)*1e6,'r',label='sigma: ' + str(sigma_e*1e3) + 'nm')
            ax2.plot(Mt,(np.ones(len(Mt))*(sigma_e_z)**2)*1e6,'--r')
            ax2.set_ylabel('nm^2');ax2.grid()
            ax2.set_xlabel('s')
            ax2.legend()
          
        # Histograms D
        """
        ax3.hist(np.mean(D,axis=0),bins=10,density=True,facecolor='b',label='M:'+ str(np.round(np.mean(np.mean(D,axis=0)),decimals=2)) +  ' STD: '+ str(np.round(np.std(np.mean(D,axis=0)),decimals=2)))
        ax3.plot(DFittParam+DBias,PDF_invGamma2,'-k',label='invGamma2')
        ax3.plot(DFittParam,PDF_invGamma,'-r',label='invGamma')
        ax3.axvline(invGammaMean2, color='g',linestyle='--',label='mean:' + str(np.round(invGammaMean2,decimals=2)))
        ax3.axvline(invGammaMode2, color='m',linestyle='--',label='mode:' + str(np.round(invGammaMode2,decimals=2)))
        ax3.axvline(invGammaMean, color='g',linestyle='-.',label='mean:' + str(np.round(invGammaMean,decimals=2)))
        ax3.axvline(invGammaMode, color='m',linestyle='-.',label='mode:' + str(np.round(invGammaMode,decimals=2)))
        ax3.legend()
   
        # Histograms S2
        ax4.hist((np.mean(S2,axis=0)-sigma_e**2)*1e6,bins=100,label='M:'+ str(np.round(np.mean((np.mean(S2,axis=0)-sigma_e**2)*1e6),decimals=0).astype('int')) + ' STD: '+ str(np.round(np.std((np.mean(S2,axis=0)-sigma_e**2)*1e6),decimals=0).astype('int')))
        ax4.legend();ax4.set_xlabel('nm^2')
        """

        if do_save_plot:
            file= Name_prefix + Name_Mid + '.svg'
            fig.savefig(path + file)
            plt.close()
        else:
            plt.show()

        print('Done!')  

def main_2():
    """
    Test of MSD and CVE for a simulated Brownian motion
    This is a addaptation of the test done in MethodAnalysis produced in 2021

    This supersides main_1 and is used for the manuscript. 
    All plots (5 of them) are saved in the "path" folder, move the plots to a folder named with the window and noise or other information used. 
    """
    do_save_plot=1
    # Path prefix for local data storage
    #prefix target folder:  20220519_(SimulationDiffusionTest)
    path = "./../"

    
    MFlags= range(5)
    
    #Simulation track
    substeps=10       #Simulate blur this will give R=1/6.2
    states=3
    D_theory=[0.1,1.4,6]    #[um^2/s]
    steps=[200,200,200]
    periodes=20
    tot_steps=periodes*np.sum(steps)
    dt=0.84e-3        #[s]
    sigma_e= [70e-3,70e-3,70e-3] #[um]
    
    # Analysis 
    #0:CVE        Dwndw>1; j=1;n=1;Flag=0
    #1: ECVE      Dwndw>2; j>2;n=1;Flag=2 Current: Dwndw=j
    #2: ECVE line fitt     j>2; Dwndw=1; n=1;Flag=2
    #3:ECVE line fitt v0   j>2; Dwndw=1; n=1;Flag=2
    #4: MSD
    Dwndw=16;



    wndw=5 #not in use
    wndwS=1 #if 1 it is a moving average, if equal to wndw it is a fix window average
    DBias=0.6   #not in use 

    #############################################################################
    # Create 3D Brownian motion 
    #############################################################################
    k=np.arange(tot_steps)
    DTrue=np.array([])
    dxdydz=np.array([[0,0,0]])
    
    for ii in range(periodes):
        for i in range(states):
            dxdydz_temp = np.random.normal(0, np.sqrt(2*D_theory[i]*(dt/substeps)), (int(steps[i]*substeps), 3))
            dxdydz=np.concatenate((dxdydz,dxdydz_temp))
            DTrue=np.concatenate((DTrue,np.ones(steps[i])*D_theory[i]))

            
    trajT = np.cumsum(dxdydz[:-1,:], axis=0)
    traj=np.zeros((tot_steps,3))
    for i in range(3):
        if substeps>1:
            traj[:,i]= np.convolve(np.ones(substeps)/substeps,trajT[:,i])[substeps-1:-substeps+1:substeps]
        else:
            traj[:,i]=trajT[:,i]
    
    XYZ = traj +  np.random.normal(np.zeros(3), sigma_e, [tot_steps,3]) #[um]
    t_track=k*dt 

    txyzn_Map=np.vstack((t_track,XYZ.T,k))

    TrajDictionary={}
    TrajToGet=0
    TrajDictionary[TrajToGet]=txyzn_Map



    ################################################################################################
    ### Estimate D and Sigma
    ################################################################################################
    for MFlag in MFlags:
        txyzn_Map=TrajDictionary[TrajToGet]
        #CVE estimation of D and sigma
        if MFlag==0:
            j=1;n=1;Flag=0
            result_txyz=_PhLib.get_CVE_MSD_traj(TrajDictionary,wndw=Dwndw,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='CVE'
        elif MFlag==1:
            j=int(Dwndw/2);n=1;Flag=2
            result_txyz=_PhLib.get_CVE_MSD_traj(TrajDictionary,wndw=Dwndw,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='ECVE'
        elif MFlag==2:
            j=Dwndw;n=1;Flag=2; jjStart=2
            result_txyz=_PhLib.get_CVE_traj_Special(TrajDictionary,wndw=1,R=1/6.2,j=j,n=n,jjStart=jjStart,Flag=Flag)
            Name_prefix='ECVE_lineFitt'
        elif MFlag==3:
            j=Dwndw;n=1;Flag=2
            result_txyz=_PhLib.get_CVE_traj_Specialv0(TrajDictionary,wndw=1,R=1/6.2,j=j,n=n,Flag=Flag)
            Name_prefix='ECVE_lineFitt0'
        elif MFlag==4:
            result_txyz=_PhLib.get_MSD_traj2(ScanProject=None,TrajDictionary=TrajDictionary,window=Dwndw,MSDStart=1,MSDSteps=Dwndw-1,PointsToUse=Dwndw-1,R=1/6.2,Flag=0)
            Name_prefix='MSD'

        t=result_txyz[str(TrajToGet),'D'][:,0]
        D=np.transpose(result_txyz[str(TrajToGet),'D'][:,1:])
        S2=np.transpose(result_txyz[str(TrajToGet),'Sigma2'][:,1:])
        #VarD=np.transpose(result_txyz[str(TrajToGet),'VarD'][:,1:])
    
        Name_Mid='_w' + str(Dwndw) + '_j'+ str(j) + '_n' + str(n) + '_F' + str(Flag)


            #Create runing mean 
        MD=np.mean(D,axis=0)
        MD = np.convolve(np.ones(wndw)/wndw,MD)
        MD=MD[wndw-1:-wndw+1:wndwS]

        MS2 = np.mean(S2,axis=0)
        MS2 = np.convolve(np.ones(wndw)/wndw,MS2)
        MS2 = MS2[wndw-1:-wndw+1:wndwS]

        Mt = np.convolve(np.ones(wndw)/wndw,t)
        Mt=Mt[wndw-1:-wndw+1:wndwS]
                
        #Fitt Gamma function 
        fit_alpha, fit_loc, fit_scale=stats.gamma.fit(np.mean(D+DBias,axis=0))
        invfit_alpha, invfit_loc, invfit_scale=stats.invgamma.fit(np.mean(D+DBias,axis=0))
        invfit_alpha2, invfit_loc2, invfit_scale2=stats.invgamma.fit(np.mean(D,axis=0))

        DFittParam=np.linspace(0,30,num=1000)
        PDF_Gamma=stats.gamma.pdf(DFittParam, a=fit_alpha,loc=fit_loc, scale=fit_scale)
        PDF_invGamma=stats.invgamma.pdf(DFittParam, a=invfit_alpha,loc=invfit_loc, scale=invfit_scale)
        PDF_invGamma2=stats.invgamma.pdf(DFittParam, a=invfit_alpha2,loc=invfit_loc2, scale=invfit_scale2)

        DFittParam=DFittParam-DBias
        fit_beta=1/fit_scale
        invfit_beta=invfit_scale 
        invfit_beta2=invfit_scale2 
        GammaMean=(fit_alpha/fit_beta)+fit_loc-DBias
        GammaMode=((fit_alpha-1)/fit_beta)+fit_loc-DBias
        invGammaMean=(invfit_beta/(invfit_alpha-1))+invfit_loc-DBias
        invGammaMode=(invfit_beta/(invfit_alpha+1))+invfit_loc-DBias
    
        invGammaMean2=(invfit_beta2/(invfit_alpha2-1))+invfit_loc2
        invGammaMode2=(invfit_beta2/(invfit_alpha2+1))+invfit_loc2

        ### Calculate bias, mean and std for each diffusion steps
        s_tot = np.sum(steps) #steps per period
        ss=np.concatenate(([0],np.cumsum(steps)))
        D_mean=np.mean(D,axis=0)
        E_mean=np.mean(S2,axis=0)*1e6 #[nm^2]

        D_dic={}
        E_dic={}
        for p in range(periodes):
            for i in range(states):
                MinMaxT=t_track[p*s_tot+ss[i]:p*s_tot+ss[i+1]][[0,-1]]
                MinMaxIndex=np.where((t>=MinMaxT[0])*(t<=MinMaxT[1]))[0][[0,-1]]
                #print('p=' + str(p) + ' s=' +  str(i) + '  MinMax=' + str(MinMaxT))
                if p==0:
                    D_dic[i]=D_mean[MinMaxIndex[0]:MinMaxIndex[1]]
                    E_dic[i]=E_mean[MinMaxIndex[0]:MinMaxIndex[1]]
                    
                else:
                    D_dic[i]=np.concatenate((D_dic[i],D_mean[MinMaxIndex[0]:MinMaxIndex[1]]))
                    E_dic[i]=np.concatenate((E_dic[i],E_mean[MinMaxIndex[0]:MinMaxIndex[1]]))
        
        D_stat={}
        E_stat={}
        for i in range(states):
            D_stat[i,'mean']=np.mean(D_dic[i])
            D_stat[i,'std']=np.std(D_dic[i])
            E_stat[i,'mean']=np.mean(E_dic[i])
            E_stat[i,'std']=np.std(E_dic[i])

        

        ###############################################################
        # Figure construction
        ###############################################################
        fig = plt.figure(constrained_layout=True)
        fig.set_size_inches(2*6, 2*5, forward=True)
        gs = GridSpec(4,4, figure=fig)
        ax0=fig.add_subplot(gs[0, 0:3]) #Trajectory 
        ax1=fig.add_subplot(gs[1, 0:3]) #Diffusion
        ax2=fig.add_subplot(gs[2, 0:3]) #Localization error 
        ax3=fig.add_subplot(gs[1, 3]) # Diffusion histogram 
        ax4=fig.add_subplot(gs[2, 3]) #Localization error  histogram
        ax5=fig.add_subplot(gs[3, 0]) #Diffusion bias, mean and std 
        ax6=fig.add_subplot(gs[3, 1]) #Info Diffusion bias, mean and std 
        ax7=fig.add_subplot(gs[3, 2]) #Localization error bias, mean and std 
        ax8=fig.add_subplot(gs[3, 3]) #Info Localization error bias, mean and std 

        ax0.set_title(Name_prefix + ' method for D and sigma')
        alpha=[0,0.4,0.8]
        S_steps=np.sum(steps)
        Cum_steps=np.concatenate(([0],np.cumsum(steps)))


        for i in range(periodes):
            if i==0:
                for ii in range(states):
                    start= i*S_steps + Cum_steps[ii]
                    end  = i*S_steps + Cum_steps[ii+1]-1
                    ax0.axvspan(txyzn_Map[0,start], txyzn_Map[0,end], alpha=alpha[ii], color='gray', label='D:'+str(D_theory[ii])+ ' t:' + str(steps[ii]*dt))
            else:
                for ii in range(states):
                    start= i*S_steps + Cum_steps[ii]
                    end  = i*S_steps + Cum_steps[ii+1]-1
                    ax0.axvspan(txyzn_Map[0,start], txyzn_Map[0,end], alpha=alpha[ii], color='gray')
        ax0.plot(txyzn_Map[0,:],txyzn_Map[1,:],'r'); ax0.plot(txyzn_Map[0,:],txyzn_Map[2,:],'g'); ax0.plot(txyzn_Map[0,:],txyzn_Map[3,:],'b')
        ax0.set_ylabel('um');ax0.grid();ax0.legend()

            
        # Diffusion constant estimation 
        #ax1.plot(t,D[0,:],'r'); ax1.plot(t,D[1,:],'g'); ax1.plot(t,D[2,:],'b')
        for i in range(periodes):
            for ii in range(states):
                start= i*S_steps + Cum_steps[ii]
                end  = i*S_steps + Cum_steps[ii+1]-1
                ax1.axvspan(txyzn_Map[0,start], txyzn_Map[0,end], alpha=alpha[ii], color='gray')

        ax1.plot(t,np.mean(D,axis=0),'k')
        #ax1.plot(t_MSD,np.mean(D_MSD,axis=0),'m')
        #ax1.plot(Mt,MD,'m') 
        ax1.plot(t_track,DTrue,'r') 
        #ax1.plot(Mt,np.ones(len(Mt))*D_theory1,'r') 
        #ax1.plot(Mt,np.ones(len(Mt))*D_theory2,'r')
        ax1.set_ylabel('um^2/s');ax1.grid()
        ax1.set_ylim(-1,20)

        # Histograms D
        #ax3.hist(np.mean(D,axis=0),bins=400,density=True,facecolor='b',label='M:'+ str(np.round(np.mean(np.mean(D,axis=0)),decimals=2)) +  ' STD: '+ str(np.round(np.std(np.mean(D,axis=0)),decimals=2)))
        for i in range(states):
            ax3.hist(D_dic[i],bins=100,density=True)
        ax3.set_xlim([-5,15])

        if MFlag!=2:
            #Localization estimation 
            #ax2.plot(t,S2[0,:],'r'); ax2.plot(t,S2[1,:],'g'); ax2.plot(t,S2[2,:],'b')
            for i in range(periodes):
                for ii in range(states):
                    start= i*S_steps + Cum_steps[ii]
                    end  = i*S_steps + Cum_steps[ii+1]-1
                    ax2.axvspan(txyzn_Map[0,start], txyzn_Map[0,end], alpha=alpha[ii], color='gray')            

            ax2.plot(t,np.mean(S2,axis=0)*1e6,'k')
            #ax2.plot(t_MSD,np.mean(S2_MSD,axis=0)*1e6,'m')
            #ax2.plot(Mt,MS2*1e6,'m')
            xyz=['x','y','z']
            for i in range(3):
                ax2.plot(Mt,(np.ones(len(Mt))*sigma_e[i]**2)*1e6,'r',label='sigma_'+ xyz[i] + ': ' + str(sigma_e[i]*1e3) + 'nm')

            ax2.set_ylabel('nm^2');ax2.grid()
            ax2.set_xlabel('s')
            ax2.legend()
             
            # Histograms S2
            #ax4.hist((np.mean(S2,axis=0)-np.mean(np.array(sigma_e)**2))*1e6,bins=100,label='M:'+ str(np.round(np.mean((np.mean(S2,axis=0)-np.mean(np.array(sigma_e)**2))*1e6),decimals=0).astype('int')) + ' STD: '+ str(np.round(np.std((np.mean(S2,axis=0)-np.mean(np.array(sigma_e)**2))*1e6),decimals=0).astype('int')))
            #ax4.legend();ax4.set_xlabel('nm^2')
            for i in range(states):
                #ax4.hist((E_dic[i]-np.mean(np.array(sigma_e)*10**3)**2),bins=100,density=True)
                ax4.hist(E_dic[i],bins=100,density=True)
            ax4.set_xlim([-15000,15000])
            ax4.set_xlabel('nm^2')
            


            # Bias, mean and Std for each diffusion step
            ax5.errorbar(D_theory, np.array([D_stat[0,'mean'],D_stat[1,'mean'],D_stat[2,'mean']])-D_theory, yerr=[D_stat[0,'std'],D_stat[1,'std'],D_stat[2,'std']], marker='*',linestyle='');
            ax5.grid(); ax5.set_ylabel('<Est>-True [um^2/s]'); ax5.set_xlabel('True [um^2/s]')
            ax5.set_ylim([-5,5])

            #Info Diffusion
            ax6.axis([0, 10, 0, 10])
            ax6.axis('off')
            for i in range(states):
                ax6.text(1,3*(3-i), 'D' + str(i) + ' = '+ str(np.round(D_stat[i,'mean'],decimals=2)) + ' +- ' + str(np.round(D_stat[i,'std'],decimals=2)) + 'um^2/s' )
                ax6.text(1,3*(3-i)-1, 'Bias' + str(i) + ' = '+ str(np.round(D_stat[i,'mean']-D_theory[i],decimals=2)) + 'um^2/s' )
            #ax6.text(0,0, str([D_stat[0,'std'],D_stat[1,'std'],D_stat[2,'std']]))
            

            # Bias, mean and Std for each noise step
            ax7.errorbar(D_theory, np.array([E_stat[0,'mean'],E_stat[1,'mean'],E_stat[2,'mean']])-(np.array(sigma_e)**2)*1e6 , yerr=[E_stat[0,'std'],E_stat[1,'std'],E_stat[2,'std']], marker='*',linestyle='');
            ax7.grid(); ax7.set_ylabel('<Est>-True  [nm^2]'); ax7.set_xlabel('True [um^2/s]')
            ax7.set_ylim([-5000,5000])

            #Info noise 
            ax8.axis([0, 10, 0, 10])
            ax8.axis('off')
            for i in range(states):
                ax8.text(0,3*(3-i), 'E' + str(i) + ' = '+ str(np.round(E_stat[i,'mean'],decimals=2)) + ' +- ' + str(np.round(E_stat[i,'std'],decimals=2)) + 'nm^2' )
                ax8.text(0,3*(3-i)-1, 'Bias' + str(i) + ' = '+ str(np.round(E_stat[i,'mean']-(np.array(sigma_e[i])**2)*1e6,decimals=2))  + 'nm^2' )

        if do_save_plot:
            file= Name_prefix + Name_Mid + '.svg'
            fig.savefig(path + file)
            plt.close()
        else:
            plt.show()

        print('Done!')  

if __name__ == '__main__':
    main_2()

    


    
   

    