import numpy as np
import matplotlib.pyplot as plt

class SpectrumClass:
    
    def get_spectrum_from_file(FileToGet):
        """
        Open file and get data that is ; separated
        """
        with open(FileToGet) as inp:
            Data=[x.strip().split(';') for x in inp]
        Data=np.array(Data)
        Data=Data.astype(np.float)
        return Data

    def SmoothingFunction(D,L,wndw,wndwStep):
        # wndwStep: if 1 then sliding window or if equal to wndw then fixwindow  
        DD = np.convolve(np.ones(wndw)/wndw,D)
        DD=DD[wndw-1:-wndw+1:wndwStep]
        Lambda=L[wndw-1::wndwStep]
        return DD,Lambda


    def PlotSpectrum(LR,SR,SRR,LT,ST,SRT,LambdaR,RM,LambdaT,TM):      
        ##Ploting
        fig, ax= plt.subplots(2,1)
        fig.set_size_inches(17.7, 10, forward=True)

            # set ticks and axis lenght 
        XaxisTicksMax=780
        XaxisTicksMin=480
        spacingX = 10
        YaxisTicksMax=1.1
        YaxisTicksMin=0
        spacingY = 0.1


            #Do the plots
        ax[0].plot(LR,SRR,
                   LT,SRT,
                   LR,SR,
                   LT,ST)
        
        ax[0].set_xlim((XaxisTicksMin,XaxisTicksMax))
        ax[0].xaxis.set_major_locator(plt.FixedLocator(np.arange(XaxisTicksMin, XaxisTicksMax, step=spacingX)))
        ax[0].yaxis.set_major_locator(plt.FixedLocator(np.arange(YaxisTicksMin, YaxisTicksMax, step=spacingY)))
        ax[0].grid()
        ax[0].set_ylabel("arb.")
        ax[0].set_xlabel("[nm]")
        ax[0].legend(('Reference','Reference','Reflected','Transmited'))

        ax[1].plot( LambdaR,RM,
                    LambdaT,TM)
        
        ax[1].grid()
        ax[1].set_ylim((YaxisTicksMin,YaxisTicksMax))
        ax[1].set_xlim((XaxisTicksMin,XaxisTicksMax))
        ax[1].xaxis.set_major_locator(plt.FixedLocator(np.arange(XaxisTicksMin, XaxisTicksMax, step=spacingX)))
        ax[1].yaxis.set_major_locator(plt.FixedLocator(np.arange(YaxisTicksMin, YaxisTicksMax, step=spacingY)))
        ax[1].set_ylabel("T/R")
        ax[1].set_xlabel("[nm]")
        ax[1].legend(('Reflectance','Transmitance'))
    
        do_save=0
        if do_save:
            fig.savefig(path + "/Filter_" + str(GetFileNumber) + ".pdf")
            plt.close()
        else:
            plt.show()        

def main_SpecturmTest():
    path="C:/Users/elias.amselem/Desktop/Shadowfax/20200520"
    fileR=[ "/FilterReflectionSpectraF5.txt",  #0
            "/FilterReflectionSpectraF16.txt", #1
            "/FilterReflectionSpectraF17.txt", #2
          ]

    fileT=[ "/FilterTransmisionSpectraF5.txt",  #0
            "/FilterTransmisionSpectraF16.txt", #1
            "/FilterTransmisionSpectraF17.txt", #2
          ]

    for GetFileNumber in [0,1,2]:
        # Get data
        S_R=SpectrumClass.get_spectrum_from_file(path+fileR[GetFileNumber])
        S_T=SpectrumClass.get_spectrum_from_file(path+fileT[GetFileNumber])
        
        LR = S_R[:,0] #Lambda       
        SR = S_R[:,1] #Signal reflected 
        SRR= S_R[:,3] #Reference reflected
        
        LT = S_T[:,0] #Lambda       
        ST = S_T[:,1] #Signal reflected 
        SRT= S_T[:,3] #Reference reflected
        
        #Calculate relative intencity to reference 
        R = SR/SRR
        T = ST/SRT

        # Smoothing, sliding window 
        wndw=10
        RM, LambdaR = SpectrumClass.SmoothingFunction(R,LR,wndw,wndwStep=1)
        TM, LambdaT = SpectrumClass.SmoothingFunction(T,LT,wndw,wndwStep=1)
        # Plot 
        SpectrumClass.PlotSpectrum(LR,SR,SRR,LT,ST,SRT,LambdaR,RM,LambdaT,TM)


    print("Done!")


if __name__ == '__main__':
    main_SpecturmTest()