from __future__ import print_function, division
import os
import torch
import re
from torch.utils.data import Dataset, DataLoader
from torchvision import transforms, utils
import numpy as np
from skimage import io
from concurrent.futures import ProcessPoolExecutor
import time
import matplotlib.pyplot as plt

from UnetSupportingFunctions.loadDataSets import mmDataSetSegment
import UnetSupportingFunctions.customTransformations as ct
from UnetSupportingFunctions import fileLoaders as fl
from UnetSupportingFunctions.mmSegmentation import mmSegmentation
from UnetSupportingFunctions.postProcessIms import cleanUpChannels
from UnetSupportingFunctions.postProcessIms import par_watershed

from UnetNetworks.PraNetUnet import PraNetUnet, ShallowPraNetUnet
from UnetNetworks.UnetPlusPlus2 import NestedUNet, UNet, ShallowNestedUNet



def Run_Segmentation(model_path,model_file,exp_dir,save_dir): 
    """
    #Trained net to use
    model_path='C:/Users/elias/Desktop/Shadowfax/ShadowPyDev/Unets/'
    model_file='ShNestedUnet_Adam_HybridLoss_512px_BF.pth'
    # Data to apply the net on
    exp_dir = 'C:/Users/elias/Desktop/Shadowfax/Data/20201208_PSF_and_CellTracking_with_markers/4_cells-MJ364/widefield_cells/'
        #Where to save the result
    save_dir = 'C:/Users/elias/Desktop/Shadowfax/Data/20201208_PSF_and_CellTracking_with_markers/4_cells-MJ364/widefieldSegmented/'
    """

    saved_model = model_path + model_file
    ims = sorted(fl.getFileList(exp_dir, chan_name='', first_only=False))

    #Transforms to convert an image to a torch format
    seg_transformations = transforms.Compose([transforms.ToPILImage(),
                                              ct.PILPadTo16(),
                                              transforms.ToTensor(),
                                              ct.SegmentationNormalize()])
    #Get list of image paths
    ims_to_segment = mmDataSetSegment(phase_ims=ims,
                                      transform=seg_transformations,
                                      output_folder=save_dir)

    segmentation_loader = DataLoader(ims_to_segment, batch_size=1, shuffle=False, num_workers=0)

    # Get the net
    net = ShallowNestedUNet(deep_supervision=True)
    saved_net = torch.load(saved_model, map_location=torch.device('cpu') )
    net.load_state_dict(saved_net['model_state_dict'])
    net.eval()
    net.cpu()

    classifier = mmSegmentation(net, threshold=0.5)
    classifier.segment(segmentation_loader)


if __name__ == '__main__':

    #Trained net to use
    model_path='C:/Users/elias/Desktop/Shadowfax/ShadowPyDev/Unets/'
    model_file='ShNestedUnet_Adam_HybridLoss_512px_BF.pth'
    # Data to apply the net on
    projectPath='C:/Users/elias/Desktop/Shadowfax/Data/20201208_PSF_and_CellTracking_with_markers/8_cells-MJ364/'
    exp_dir = projectPath + 'widefield_cells/'
    #Where to save the result
    save_dir = projectPath + 'widefieldSegmented/'

    Run_Segmentation(model_path,model_file,exp_dir,save_dir)
    print("Done!")
