import torch
from torch.autograd import Variable
import torch.nn.functional as F
from torch.utils.data import DataLoader
import numpy as np
from skimage import io, transform, filters, segmentation, measure, feature, morphology, util
from scipy import ndimage as ndi
import matplotlib.pyplot as plt

class mmSegmentation:
    
    """ trainer for microscopy experiment UNet. 
    The classifier will run 1 epoch, then test the accuracy on test data."""
    
    def __init__(self,net,threshold=0.5):
        """
        The classifier for mm experiment
            net (nn.Module): The neural net module containing the definition of your model
            num_epochs (int): The maximum number of epochs on which the model will train
        """
        self.net = net
        self.use_cuda = torch.cuda.is_available()
        self.threshold = threshold
        #self.sz = sz

    def segment(self, seg_loader: DataLoader):
        
        for sample in seg_loader:
            im, out_path = sample['im'], sample['path_to_save']
            sz=sample['imsize'].numpy()[0]

            pred_o = self.net(im)

            if len(pred_o)>1: #if deep supervision, use final prediction
                pred_o=pred_o[-1]
            
            pred = torch.sigmoid(pred_o) #remaping image values to the range of 0 to 1, this also compresses the values close to 0 or negative to be close to zero and the same for the larger values but this compreses close to 1
            pred = pred.to('cpu').detach().numpy().squeeze(0).squeeze(0) #Convert ot numpy
            pred = pred[0:sz[0], 0:sz[1]] #Reshape 
            
            if 0: #Debug
                fig, ax= plt.subplots(1,4)
                ax[0].imshow(im[0,0,:,:])
                ax[1].imshow(pred_o.detach().numpy().squeeze(0).squeeze(0))
                ax[2].imshow(pred)
                ax[3].imshow(pred > self.threshold)
                plt.show()
   
            thresh = self.threshold
            pred = pred > self.threshold
            pred = morphology.remove_small_objects(pred,10)
            pred_labels = measure.label(pred).astype('uint16') #Find patches that are conected and set a number, with this each patch is "named"

            if 0: #Debug
                fig, ax= plt.subplots(1,4)
                ax[0].imshow(im[0,0,:,:])
                ax[1].imshow(pred_o.detach().numpy().squeeze(0).squeeze(0))
                ax[2].imshow(pred)
                ax[3].imshow(pred_labels)
                plt.show()

            io.imsave(out_path[0],pred_labels,compress=6,check_contrast=False)
                 
