import numpy as np 
import vtk 
import vtk.util.numpy_support as npsup

class VTKgraphPlotLib:
    def main_first():
        # Create the geometry of a point (the coordinate)
        points = vtk.vtkPoints()
        p = [1.0, 2.0, 3.0]

        # Create the topology of the point (a vertex)
        vertices = vtk.vtkCellArray()

        id = points.InsertNextPoint(p)
        vertices.InsertNextCell(1)
        vertices.InsertCellPoint(id)

        # Create a polydata object
        point = vtk.vtkPolyData()

        # Set the points and vertices we created as the geometry and topology of the polydata
        point.SetPoints(points)
        point.SetVerts(vertices)

        # Visualize
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInputData(point)

        # Create actor
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        actor.GetProperty().SetPointSize(20)
        actor.GetProperty().SetColor(1.0, 0.0, 0.0)

        # Create renderer 
        renderer = vtk.vtkRenderer()
        renderer.SetBackground(0.0, 0.0, 0.0) 
        renderer.AddActor(actor)

        # Create render window 
        renderWindow = vtk.vtkRenderWindow()
        renderWindow.SetWindowName("Simple VTK scene")
        renderWindow.SetSize(400, 400) 
        renderWindow.AddRenderer(renderer)

        #Create an interactor 
        renderWindowInteractor = vtk.vtkRenderWindowInteractor()
        renderWindowInteractor.SetRenderWindow(renderWindow)

        # Initialize the interactor and start the # rendering loop
        renderWindowInteractor.Initialize() 
        renderWindow.Render()
        renderWindowInteractor.Start()

    def main_points():
        colors = vtk.vtkNamedColors()

        # create a rendering window and renderer
        ren = vtk.vtkRenderer()
        renWin = vtk.vtkRenderWindow()
        renWin.AddRenderer(ren)

        # create a renderwindowinteractor
        iren = vtk.vtkRenderWindowInteractor()
        iren.SetRenderWindow(renWin)

        # create source
        src = vtk.vtkPointSource()
        src.SetCenter(0, 0, 0)
        src.SetNumberOfPoints(500)
        src.SetRadius(10)
        src.Update()
    
        # mapper
        mapper = vtk.vtkPolyDataMapper()
        mapper.SetInputConnection(src.GetOutputPort())

        # actor
        actor = vtk.vtkActor()
        actor.SetMapper(mapper)
        actor.GetProperty().SetColor(colors.GetColor3d("Tomato"))
        actor.GetProperty().SetPointSize(4)

        # assign actor to the renderer
        ren.AddActor(actor)
        ren.SetBackground(colors.GetColor3d("DarkGreen"))

        # enable user interface interactor
        iren.Initialize()
        renWin.Render()
        iren.Start()


if __name__ == '__main__':
    VTKgraphPlotLib.main_points()



"""
Code below are exampels for further development 
# Functions that make vtk object out of XYZ locations.

def makeCylinderPtsVTP(locXYZ,radius=50,height=50,res=10):

    # Load the file

    if type(locXYZ) == np.ndarray:

        loc = locXYZ

    elif type(locXYZ) == str:

        loc = np.genfromtxt(locXYZ)

    # Make append poly filter

    appPoly = vtk.vtkAppendPolyData()

    # Loop through all the locations

    for pt in loc[:,0:3]:

    	# Make the cylinters

        cyl = vtk.vtkCylinderSource()

        cyl.SetCenter(pt)

        cyl.SetRadius(radius)

        cyl.SetHeight(height)

        cyl.SetResolution(res)

        # Rotate to be vertical

        rot = vtk.vtkTransform()

        rot.Translate(-pt)

        rot.PostMultiply()

        rot.RotateX(90)

        rot.Translate(pt)

        tranFor = vtk.vtkTransformPolyDataFilter()

        tranFor.SetInputConnection(cyl.GetOutputPort())

        tranFor.SetTransform(rot)

        tranFor.Update()

        # Append

        appPoly.AddInputConnection(tranFor.GetOutputPort())

    # Update and return.

    appPoly.Update()

    return appPoly.GetOutput()



def makeSpherePtsVTP(locXYZ,radius=50,res=20):

    # Load the file

    if type(locXYZ) == np.ndarray:

        loc = locXYZ

    elif type(locXYZ) == str:

        loc = np.genfromtxt(locXYZ)

    # Make append poly filter

    appPoly = vtk.vtkAppendPolyData()

    # Loop through all the locations

    for pt in loc[:,0:3]:

    	# Make the spheres

        sph = vtk.vtkSphereSource()

        sph.SetCenter(pt)

        sph.SetRadius(radius)

        sph.SetPhiResolution(res)

        sph.SetThetaResolution(res)

        sph.Update()

        # Append

        appPoly.AddInputConnection(sph.GetOutputPort())

    # Update and return.

    appPoly.Update()

    return appPoly.GetOutput()



def makeCubePtsVTP(locXYZ,xdim=50,ydim=50,zdim=50):

    # Load the file

    if type(locXYZ) == np.ndarray:

        loc = locXYZ

    elif type(locXYZ) == str:

        loc = np.genfromtxt(locXYZ)

    # Make append poly filter

    appPoly = vtk.vtkAppendPolyData()

    # Loop through all the locations

    for pt in loc[:,0:3]:

    	# Make the cubes

        cube = vtk.vtkCubeSource()

        cube.SetCenter(pt)

        cube.SetXLength(xdim)

        cube.SetYLength(ydim)

        cube.SetZLength(zdim)

        cube.Update()

        # Append

        appPoly.AddInputConnection(cube.GetOutputPort())

    # Update and return.

    appPoly.Update()

    return appPoly.GetOutput()



def makeSurfaceVTP(locXYZ):

    # Load the file

    if type(locXYZ) == np.ndarray:

        loc = locXYZ

    elif type(locXYZ) == str:

        loc = np.genfromtxt(locXYZ)



    # Make the pts

    vtkPts = vtk.vtkPoints()

    vtkPts.SetData(npsup.numpy_to_vtk(loc,deep=1))

    # Make the poly data

    polyPtsVTP = vtk.vtkPolyData()

    polyPtsVTP.SetPoints(vtkPts)

    # Triangulate

    del2Filt = vtk.vtkDelaunay2D()

    del2Filt.SetInputData(polyPtsVTP)

    del2Filt.Update()

    # Return

    return del2Filt.GetOutput()
"""