"""
This module is used to generate analytical PSFs. Currently only gaussian
staircases are implemented
"""

import numpy as np
from ConfigObject_PhRetrieval import ConfigObject_PhRetrieval as _CPhR

class AnalyticalPSF:
    def create_Gaussian_staircase(parm, depth_spacing=0.235, sigma, spot_num=5):
        """
        Creates an analytical, gaussian approximation of the staircase PSF.
        Returns an NxMxO size grid, which are determined from the config file.
        """
        xx, yy, zz = AnalyticalPSF.getXYZGrid(parm, depth_spacing*spot_num)
        # generate spot locations
        # u = np.cos(2*np.pi*np.arange(spot_num)/spot_num)
        # v = np.sin(2*np.pi*np.arange(spot_num)/spot_num)
        # w = depth_spacing*np.roll(np.arange(spot_num) - np.floor(spot_num/2), -int(np.floor(spot_num/2)))

        # define the standard 
        sigma_xy = sigma
        sigma_z = sigma_xy * 2

        gauss_PSF = np.zeros(np.size(xx))
        for i in range(spot_num):
            gauss_PSF += np.exp(-np.square(xx - u[i])/sigma_xy**2 - np.square(yy - v[i])/sigma_xy**2 - np.square(zz - w[i])/sigma_z**2)

        return gauss_PSF


    @staticmethod
    def getXYZGrid(param, zheight):
        # get pixel Coordinates in [um]
        Px     = param['Px']*1e6 #[um] Andor Ixon 897(512x512) pixel size is 16um, for Ixon 888 (1024x1024) it is 13um
        Mag    = param['Mag']    # Magnification M=f_tube/f_objective
        Mag2   = param['Mag2']   # Secondary magnification to EMCCD
        NM     = param['NM']     # Cropped image pixel dimensions 
        ZSWScalefactor = param['ZSWScalefactor'] # Software scaling factor
        zssize = param['zssize'] #[m] z- step size between images
        
        N = NM[0]
        M = NM[1]
        O = zheight/(zssize/ZSWScalefactor) # pixel count = (total height)/(height per pixel)
        
        #
        x = np.linspace(-(Px/(Mag*Mag2))*N/2,(Px/(Mag*Mag2))*N/2, N)
        y = np.linspace(-(Px/(Mag*Mag2))*M/2,(Px/(Mag*Mag2))*M/2, M)
        z = np.linspace(-zheight/2, zheight/2, O)
        xx, yy, zz = np.meshgrid(x, y, z)
        return xx, yy, zz

def main():
    # path = "C:/Users/elias.amselem/Desktop/Shadowfax/20200618/processed/"
    path = "C:/Users/bobro/Documents/ShadowDev/Data/20200618/processed/"
    configFile = 'AnalysisConfig_100x100x30crop.ini'

    # Get configuration settings
    SettingHandler = _CPhR(configfile= path+configFile)
    SettingHandler.get_section_Ph_Retrieval()
    params = SettingHandler.Ph_Retrieval

    # Get experimental settings to calculate the standard deviation
    lambda = 540e-9
    NA = 1.45
    sigma = (0.822/2)*(lambda/NA)*1e6  #[um] theoretical PSF width, the radius of the Airy disk is scaled to 1.22/2 and for the e-2 intensity drop that is std value one use 0.822/2